@@ -1,6 +1,6 @@
 /* gcal-event.c
  *
- * Copyright (C) 2016 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
+ * Copyright (C) 2016-2020 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,6 +89,7 @@ struct _GcalEvent
 
   GDateTime          *dt_start;
   GDateTime          *dt_end;
+  GcalRange          *range;
 
   GdkRGBA            *color;
   GBinding           *color_binding;
@@ -124,6 +125,7 @@ enum {
   PROP_DATE_START,
   PROP_LOCATION,
   PROP_CALENDAR,
+  PROP_RANGE,
   PROP_SUMMARY,
   PROP_TIMEZONE,
   PROP_UID,
@@ -134,41 +136,17 @@ enum {
 
 static GParamSpec* properties[N_PROPS] = { NULL, };
 
-
-/*
- * GcalEvent cache
- */
-
-static GHashTable *event_cache = NULL;
-
-G_DEFINE_CONSTRUCTOR (init_event_cache_map);
-
-static void
-init_event_cache_map (void)
-{
-  event_cache = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
-}
-
-G_DEFINE_DESTRUCTOR (destroy_event_cache_map);
-
-static void
-destroy_event_cache_map (void)
-{
-  GList *events;
-
-  g_debug ("Number of cached events at destruction: %d", g_hash_table_size (event_cache));
-
-  /* Destroy all events */
-  events = g_hash_table_get_values (event_cache);
-  g_list_free_full (events, g_object_unref);
-
-  g_hash_table_destroy (event_cache);
-}
-
 /*
  * Auxiliary methods
  */
 
+static void
+clear_range (GcalEvent *self)
+{
+  g_clear_pointer (&self->range, gcal_range_unref);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_RANGE]);
+}
+
 static GTimeZone*
 get_timezone_from_ical (ECalComponentDateTime *comp)
 {
@@ -264,21 +242,11 @@ gcal_event_update_uid_internal (GcalEvent *self)
 {
   ECalComponentId *id;
   const gchar *source_id;
-  gboolean should_update_cache;
 
   /* Setup event uid */
   source_id = self->calendar ? gcal_calendar_get_id (self->calendar) : "";
   id = e_cal_component_get_id (self->component);
 
-  should_update_cache = self->uid != NULL;
-
-  if (should_update_cache)
-    {
-      g_debug ("Removing '%s' (%p) from cache", self->uid, self);
-      g_hash_table_remove (event_cache, self->uid);
-      g_clear_pointer (&self->uid, g_free);
-    }
-
   if (e_cal_component_id_get_rid (id) != NULL)
     {
       self->uid = g_strdup_printf ("%s:%s:%s",
@@ -293,12 +261,6 @@ gcal_event_update_uid_internal (GcalEvent *self)
                                    e_cal_component_id_get_uid (id));
     }
 
-  if (should_update_cache)
-    {
-      g_debug ("Adding %s to the cache", self->uid);
-      g_hash_table_insert (event_cache, self->uid, self);
-    }
-
   e_cal_component_id_free (id);
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_UID]);
 }
@@ -336,125 +298,124 @@ static void
 gcal_event_set_component_internal (GcalEvent     *self,
                                    ECalComponent *component)
 {
-  if (g_set_object (&self->component, component))
+  g_autoptr (GTimeZone) zone_start = NULL;
+  g_autoptr (GTimeZone) zone_end = NULL;
+  ECalComponentDateTime *start;
+  ECalComponentDateTime *end;
+  ECalComponentText *text;
+  ICalTime *date;
+  GDateTime *date_start;
+  GDateTime *date_end;
+  gboolean start_is_all_day, end_is_all_day;
+  gchar *description, *location;
+
+  g_assert (self->component == NULL);
+
+  self->component = e_cal_component_clone (component);
+
+  /* Setup start date */
+  start = e_cal_component_get_dtstart (component);
+
+  /*
+   * A NULL start date is invalid. We set something bogus to proceed, and make
+   * it set a GError and return NULL.
+   */
+  if (!start || !e_cal_component_datetime_get_value (start))
     {
-      ECalComponentDateTime *start;
-      ECalComponentDateTime *end;
-      ECalComponentText *text;
-      ICalTime *date;
-      GDateTime *date_start;
-      GTimeZone *zone_start = NULL;
-      GDateTime *date_end;
-      GTimeZone *zone_end = NULL;
-      gboolean start_is_all_day, end_is_all_day;
-      gchar *description, *location;
+      self->is_valid = FALSE;
+      g_set_error (&self->initialization_error,
+                   GCAL_EVENT_ERROR,
+                   GCAL_EVENT_ERROR_INVALID_START_DATE,
+                   "Event '%s' has an invalid start date", gcal_event_get_uid (self));
 
-      /* Setup start date */
-      start = e_cal_component_get_dtstart (component);
+      e_cal_component_datetime_free (start);
+      start = e_cal_component_datetime_new_take (i_cal_time_new_today (), NULL);
+    }
 
-      /*
-       * A NULL start date is invalid. We set something bogus to proceed, and make
-       * it set a GError and return NULL.
-       */
-      if (!start || !e_cal_component_datetime_get_value (start))
-        {
-          self->is_valid = FALSE;
-          g_set_error (&self->initialization_error,
-                       GCAL_EVENT_ERROR,
-                       GCAL_EVENT_ERROR_INVALID_START_DATE,
-                       "Event '%s' has an invalid start date", gcal_event_get_uid (self));
+  GCAL_TRACE_MSG ("Retrieving start timezone");
 
-          e_cal_component_datetime_free (start);
-          start = e_cal_component_datetime_new_take (i_cal_time_new_today (), NULL);
-        }
+  date = i_cal_time_normalize (e_cal_component_datetime_get_value (start));
+  zone_start = get_timezone_from_ical (start);
+  date_start = g_date_time_new (zone_start,
+                                i_cal_time_get_year (date),
+                                i_cal_time_get_month (date),
+                                i_cal_time_get_day (date),
+                                i_cal_time_is_date (date) ? 0 : i_cal_time_get_hour (date),
+                                i_cal_time_is_date (date) ? 0 : i_cal_time_get_minute (date),
+                                i_cal_time_is_date (date) ? 0 : i_cal_time_get_second (date));
+  start_is_all_day = gcal_date_time_is_date (date_start);
 
-      GCAL_TRACE_MSG ("Retrieving start timezone");
+  self->dt_start = date_start;
 
-      date = i_cal_time_normalize (e_cal_component_datetime_get_value (start));
-      zone_start = get_timezone_from_ical (start);
-      date_start = g_date_time_new (zone_start,
-                                    i_cal_time_get_year (date),
-                                    i_cal_time_get_month (date),
-                                    i_cal_time_get_day (date),
-                                    i_cal_time_is_date (date) ? 0 : i_cal_time_get_hour (date),
-                                    i_cal_time_is_date (date) ? 0 : i_cal_time_get_minute (date),
-                                    i_cal_time_is_date (date) ? 0 : i_cal_time_get_second (date));
-      start_is_all_day = gcal_date_time_is_date (date_start);
+  g_clear_object (&date);
 
-      self->dt_start = date_start;
+  /* Setup end date */
+  end = e_cal_component_get_dtend (component);
+
+  if (!end || !e_cal_component_datetime_get_value (end))
+    {
+      self->all_day = TRUE;
+    }
+  else
+    {
+      GCAL_TRACE_MSG ("Retrieving end timezone");
+
+      date = i_cal_time_normalize (e_cal_component_datetime_get_value (end));
+      zone_end = get_timezone_from_ical (end);
+      date_end = g_date_time_new (zone_end,
+                                  i_cal_time_get_year (date),
+                                  i_cal_time_get_month (date),
+                                  i_cal_time_get_day (date),
+                                  i_cal_time_is_date (date) ? 0 : i_cal_time_get_hour (date),
+                                  i_cal_time_is_date (date) ? 0 : i_cal_time_get_minute (date),
+                                  i_cal_time_is_date (date) ? 0 : i_cal_time_get_second (date));
+      end_is_all_day = gcal_date_time_is_date (date_end);
+
+      self->dt_end = g_date_time_ref (date_end);
+
+      /* Setup all day */
+      self->all_day = start_is_all_day && end_is_all_day;
 
       g_clear_object (&date);
-
-      /* Setup end date */
-      end = e_cal_component_get_dtend (component);
-
-      if (!end || !e_cal_component_datetime_get_value (end))
-        {
-          self->all_day = TRUE;
-        }
-      else
-        {
-          GCAL_TRACE_MSG ("Retrieving end timezone");
-
-          date = i_cal_time_normalize (e_cal_component_datetime_get_value (end));
-          zone_end = get_timezone_from_ical (end);
-          date_end = g_date_time_new (zone_end,
-                                      i_cal_time_get_year (date),
-                                      i_cal_time_get_month (date),
-                                      i_cal_time_get_day (date),
-                                      i_cal_time_is_date (date) ? 0 : i_cal_time_get_hour (date),
-                                      i_cal_time_is_date (date) ? 0 : i_cal_time_get_minute (date),
-                                      i_cal_time_is_date (date) ? 0 : i_cal_time_get_second (date));
-          end_is_all_day = gcal_date_time_is_date (date_end);
-
-          self->dt_end = g_date_time_ref (date_end);
-
-          /* Setup all day */
-          self->all_day = start_is_all_day && end_is_all_day;
-
-          g_clear_object (&date);
-        }
-
-      /* Summary */
-      text = e_cal_component_get_summary (component);
-      if (text && e_cal_component_text_get_value (text))
-        gcal_event_set_summary (self, e_cal_component_text_get_value (text));
-      else
-        gcal_event_set_summary (self, "");
-
-      /* Location */
-      location = e_cal_component_get_location (component);
-      gcal_event_set_location (self, location ? location : "");
-
-      /* Setup description */
-      description = get_desc_from_component (component, "\n\n");
-      gcal_event_set_description (self, description);
-
-      /* Setup UID */
-      gcal_event_update_uid_internal (self);
-
-      /* Set has-recurrence to check if the component has recurrence or not */
-      self->has_recurrence = e_cal_component_has_recurrences(component);
-
-      /* Load the recurrence-rules in GcalRecurrence struct */
-      self->recurrence = gcal_recurrence_parse_recurrence_rules (component);
-
-      /* Load and setup the alarms */
-      load_alarms (self);
-
-      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HAS_RECURRENCE]);
-      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_RECURRENCE]);
-      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMPONENT]);
-
-      g_clear_pointer (&zone_start, g_time_zone_unref);
-      g_clear_pointer (&zone_end, g_time_zone_unref);
-      g_clear_pointer (&description, g_free);
-      g_clear_pointer (&location, g_free);
-
-      e_cal_component_text_free (text);
-      e_cal_component_datetime_free (start);
-      e_cal_component_datetime_free (end);
     }
+
+  /* Summary */
+  text = e_cal_component_get_summary (component);
+  if (text && e_cal_component_text_get_value (text))
+    gcal_event_set_summary (self, e_cal_component_text_get_value (text));
+  else
+    gcal_event_set_summary (self, "");
+
+  /* Location */
+  location = e_cal_component_get_location (component);
+  gcal_event_set_location (self, location ? location : "");
+
+  /* Setup description */
+  description = get_desc_from_component (component, "\n\n");
+  gcal_event_set_description (self, description);
+
+  /* Setup UID */
+  gcal_event_update_uid_internal (self);
+
+  /* Set has-recurrence to check if the component has recurrence or not */
+  self->has_recurrence = e_cal_component_has_recurrences(component);
+
+  /* Load the recurrence-rules in GcalRecurrence struct */
+  self->recurrence = gcal_recurrence_parse_recurrence_rules (component);
+
+  /* Load and setup the alarms */
+  load_alarms (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HAS_RECURRENCE]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_RECURRENCE]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMPONENT]);
+
+  g_clear_pointer (&description, g_free);
+  g_clear_pointer (&location, g_free);
+
+  e_cal_component_text_free (text);
+  e_cal_component_datetime_free (start);
+  e_cal_component_datetime_free (end);
 }
 
 /*
@@ -493,12 +454,9 @@ gcal_event_finalize (GObject *object)
 {
   GcalEvent *self = (GcalEvent *)object;
 
-  g_debug ("Removing '%s' (%p) from cache", self->uid, self);
-
-  g_hash_table_remove (event_cache, self->uid);
-
   g_clear_pointer (&self->dt_start, g_date_time_unref);
   g_clear_pointer (&self->dt_end, g_date_time_unref);
+  g_clear_pointer (&self->range, gcal_range_unref);
   g_clear_pointer (&self->summary, g_free);
   g_clear_pointer (&self->location, g_free);
   g_clear_pointer (&self->description, g_free);
@@ -554,6 +512,10 @@ gcal_event_get_property (GObject    *object,
       g_value_set_string (value, gcal_event_get_location (self));
       break;
 
+    case PROP_RANGE:
+      g_value_take_boxed (value, gcal_event_get_range (self));
+      break;
+
     case PROP_SUMMARY:
       g_value_set_string (value, gcal_event_get_summary (self));
       break;
@@ -728,6 +690,17 @@ gcal_event_class_init (GcalEventClass *klass)
                                                    "",
                                                    G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
+  /**
+  * GcalEvent::range:
+  *
+  * The range of the event, as a combination of the start and end dates.
+  */
+  properties[PROP_RANGE] = g_param_spec_boxed ("range",
+                                               "Range of the event",
+                                               "The time range of the event",
+                                               GCAL_TYPE_RANGE,
+                                               G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
   /**
   * GcalEvent::recurrence:
   *
@@ -818,36 +791,12 @@ gcal_event_new (GcalCalendar   *calendar,
                 ECalComponent  *component,
                 GError        **error)
 {
-  GcalEvent *event;
-  g_autofree gchar *uuid;
-
-  uuid = get_uuid_from_component (gcal_calendar_get_source (calendar), component);
-
-  if (g_hash_table_contains (event_cache, uuid))
-    {
-      g_debug ("Using cached value for %s", uuid);
-
-      event = g_hash_table_lookup (event_cache, uuid);
-      gcal_event_set_component_internal (event, component);
-      g_object_ref (event);
-    }
-  else
-    {
-      event = g_initable_new (GCAL_TYPE_EVENT,
-                              NULL,
-                              error,
-                              "calendar", calendar,
-                              "component", component,
-                              NULL);
-
-      if (event)
-        {
-          g_debug ("Adding %s to the cache", event->uid);
-          g_hash_table_insert (event_cache, event->uid, event);
-        }
-    }
-
-  return event;
+  return g_initable_new (GCAL_TYPE_EVENT,
+                         NULL,
+                         error,
+                         "calendar", calendar,
+                         "component", component,
+                         NULL);
 }
 
 /**
@@ -939,6 +888,8 @@ gcal_event_set_all_day (GcalEvent *self,
       self->all_day = all_day;
 
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ALL_DAY]);
+
+      clear_range (self);
     }
 }
 
@@ -974,13 +925,10 @@ gcal_event_set_date_end (GcalEvent *self,
 {
   g_return_if_fail (GCAL_IS_EVENT (self));
 
-  if (self->dt_end != dt)
+  if (gcal_set_date_time (&self->dt_end, dt))
     {
       ECalComponentDateTime *component_dt;
 
-      g_clear_pointer (&self->dt_end, g_date_time_unref);
-      self->dt_end = g_date_time_ref (dt);
-
       /* Setup the ECalComponent's datetime value */
       component_dt = build_component_from_datetime (self, dt);
 
@@ -989,6 +937,7 @@ gcal_event_set_date_end (GcalEvent *self,
 
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DATE_END]);
 
+      clear_range (self);
       e_cal_component_datetime_free (component_dt);
     }
 }
@@ -1022,13 +971,10 @@ gcal_event_set_date_start (GcalEvent *self,
 {
   g_return_if_fail (GCAL_IS_EVENT (self));
 
-  if (self->dt_start != dt)
+  if (gcal_set_date_time (&self->dt_start, dt))
     {
       ECalComponentDateTime *component_dt;
 
-      g_clear_pointer (&self->dt_start, g_date_time_unref);
-      self->dt_start = g_date_time_ref (dt);
-
       /* Setup the ECalComponent's datetime value */
       component_dt = build_component_from_datetime (self, dt);
 
@@ -1037,10 +983,35 @@ gcal_event_set_date_start (GcalEvent *self,
 
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DATE_START]);
 
+      clear_range (self);
       e_cal_component_datetime_free (component_dt);
     }
 }
 
+/**
+ * gcal_event_get_range:
+ * @self: a #GcalEvent
+ *
+ * Retrieves the range of @self. This is a simple combination of
+ * the start and end dates.
+ *
+ * Returns: (transfer none): a #GcalRange
+ */
+GcalRange*
+gcal_event_get_range (GcalEvent *self)
+{
+  g_return_val_if_fail (GCAL_IS_EVENT (self), NULL);
+
+  if (!self->range)
+    {
+      self->range = gcal_range_new (gcal_event_get_date_start (self),
+                                    gcal_event_get_date_end (self),
+                                    self->all_day ? GCAL_RANGE_DATE_ONLY : GCAL_RANGE_DEFAULT);
+    }
+
+  return self->range;
+}
+
 /**
  * gcal_event_get_description:
  * @self a #GcalEvent
