@@ -1,6 +1,6 @@
 /* gcal-event.h
  *
- * Copyright (C) 2016 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
+ * Copyright (C) 2016-2020 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #define GCAL_EVENT_H
 
 #include "gcal-calendar.h"
+#include "gcal-range.h"
 #include "gcal-recurrence.h"
 
 #include <glib-object.h>
@@ -74,6 +75,8 @@ GDateTime*           gcal_event_get_date_start                   (GcalEvent
 void                 gcal_event_set_date_start                   (GcalEvent          *self,
                                                                   GDateTime          *dt);
 
+GcalRange*           gcal_event_get_range                        (GcalEvent          *self);
+
 const gchar*         gcal_event_get_description                  (GcalEvent          *self);
 
 void                 gcal_event_set_description                  (GcalEvent          *self,
