@@ -22,6 +22,8 @@
 #include "gcal-context.h"
 #include "gcal-debug.h"
 #include "gcal-manager.h"
+#include "gcal-timeline.h"
+#include "gcal-timeline-subscriber.h"
 #include "gcal-utils.h"
 
 #include <libedataserverui/libedataserverui.h>
@@ -43,16 +45,6 @@ typedef struct
   GcalManager        *manager;
 } AsyncOpsData;
 
-typedef struct
-{
-  ECalDataModelSubscriber *subscriber;
-  gchar              *query;
-
-  guint               sources_left;
-  gboolean            passed_start;
-  gboolean            search_done;
-} ViewStateData;
-
 typedef struct
 {
   gchar              *event_uid;
@@ -62,12 +54,6 @@ typedef struct
   GcalManager        *manager;
 } MoveEventData;
 
-typedef struct
-{
-  GcalManager        *manager;
-  GList              *events;
-} GatherEventData;
-
 struct _GcalManager
 {
   GObject             parent;
@@ -82,15 +68,12 @@ struct _GcalManager
   ESourceRegistry    *source_registry;
   ECredentialsPrompter *credentials_prompter;
 
-  ECalDataModel      *e_data_model;
-
-  ECalDataModel      *shell_search_data_model;
-  ViewStateData      *search_view_data;
-
   GCancellable       *async_ops;
 
   gint                clients_synchronizing;
 
+  GcalTimeline       *timeline;
+
   GcalContext        *context;
 };
 
@@ -110,7 +93,6 @@ enum
   CALENDAR_ADDED,
   CALENDAR_CHANGED,
   CALENDAR_REMOVED,
-  QUERY_COMPLETED,
   NUM_SIGNALS
 };
 
@@ -124,36 +106,6 @@ free_async_ops_data (AsyncOpsData *data)
   g_free (data);
 }
 
-static gboolean
-gather_events (ECalDataModel         *data_model,
-               ECalClient            *client,
-               const ECalComponentId *id,
-               ECalComponent         *comp,
-               time_t                 instance_start,
-               time_t                 instance_end,
-               gpointer               user_data)
-{
-  GatherEventData *data = user_data;
-  GcalCalendar *calendar;
-  GcalEvent *event;
-  GError *error;
-
-  error = NULL;
-  calendar = g_hash_table_lookup (data->manager->clients, e_client_get_source (E_CLIENT (client)));
-  event = gcal_event_new (calendar, comp, &error);
-
-  if (error)
-    {
-      g_warning ("Error: %s", error->message);
-      g_clear_error (&error);
-      return TRUE;
-    }
-
-  data->events = g_list_append (data->events, event);/* FIXME: add me sorted */
-
-  return TRUE;
-}
-
 static void
 remove_source (GcalManager  *self,
                ESource      *source)
@@ -176,9 +128,7 @@ remove_source (GcalManager  *self,
 
   g_object_ref (calendar);
 
-  e_cal_data_model_remove_client (self->e_data_model,
-                                  e_source_get_uid (source));
-
+  gcal_timeline_remove_calendar (self->timeline, calendar);
   g_hash_table_remove (self->clients, source);
   g_signal_emit (self, signals[CALENDAR_REMOVED], 0, calendar);
 
@@ -198,23 +148,6 @@ source_changed (GcalManager *self,
   if (!calendar)
     GCAL_RETURN ();
 
-  if (gcal_calendar_get_visible (calendar))
-    {
-      ECalClient *client = gcal_calendar_get_client (calendar);
-
-      e_cal_data_model_add_client (self->e_data_model, client);
-      if (self->shell_search_data_model)
-        e_cal_data_model_add_client (self->shell_search_data_model, client);
-    }
-  else
-    {
-      const gchar *id = gcal_calendar_get_id (calendar);
-
-      e_cal_data_model_remove_client (self->e_data_model, id);
-      if (self->shell_search_data_model)
-        e_cal_data_model_remove_client (self->shell_search_data_model, id);
-    }
-
   g_signal_emit (self, signals[CALENDAR_CHANGED], 0, calendar);
 
   GCAL_EXIT;
@@ -286,12 +219,7 @@ on_calendar_created_cb (GObject      *source_object,
 
   g_hash_table_insert (self->clients, g_object_ref (source), calendar);
 
-  if (visible)
-    {
-      e_cal_data_model_add_client (self->e_data_model, client);
-      if (self->shell_search_data_model != NULL)
-        e_cal_data_model_add_client (self->shell_search_data_model, client);
-    }
+  gcal_timeline_add_calendar (self->timeline, calendar);
 
   /* refresh client when it's added */
   if (visible && e_client_check_refresh_supported (E_CLIENT (client)))
@@ -465,45 +393,6 @@ on_event_removed (GObject      *source_object,
   GCAL_EXIT;
 }
 
-static void
-model_state_changed (GcalManager            *self,
-                     ECalClientView         *view,
-                     ECalDataModelViewState  state,
-                     guint                   percent,
-                     const gchar            *message,
-                     const GError           *error,
-                     ECalDataModel          *data_model)
-{
-  gchar *filter;
-
-  GCAL_ENTRY;
-
-  filter = e_cal_data_model_dup_filter (data_model);
-
-  if (state == E_CAL_DATA_MODEL_VIEW_STATE_START &&
-      g_strcmp0 (self->search_view_data->query, filter) == 0)
-    {
-      self->search_view_data->passed_start = TRUE;
-      GCAL_GOTO (out);
-    }
-
-  if (self->search_view_data->passed_start &&
-      state == E_CAL_DATA_MODEL_VIEW_STATE_COMPLETE &&
-      g_strcmp0 (self->search_view_data->query, filter) == 0)
-    {
-      self->search_view_data->sources_left--;
-      self->search_view_data->search_done = (self->search_view_data->sources_left == 0);
-
-      if (self->search_view_data->search_done)
-        g_signal_emit (self, signals[QUERY_COMPLETED], 0);
-    }
-
-out:
-  g_free (filter);
-
-  GCAL_EXIT;
-}
-
 static void
 show_source_error (const gchar  *where,
                    const gchar  *what,
@@ -653,8 +542,7 @@ gcal_manager_finalize (GObject *object)
 
   GCAL_ENTRY;
 
-  g_clear_object (&self->e_data_model);
-  g_clear_object (&self->shell_search_data_model);
+  g_clear_object (&self->timeline);
 
   if (self->context)
     {
@@ -662,12 +550,6 @@ gcal_manager_finalize (GObject *object)
       self->context = NULL;
     }
 
-  if (self->search_view_data)
-    {
-      g_clear_pointer (&self->search_view_data->query, g_free);
-      g_clear_pointer (&self->search_view_data, g_free);
-    }
-
   g_clear_pointer (&self->clients, g_hash_table_destroy);
 
   G_OBJECT_CLASS (gcal_manager_parent_class)->finalize (object);
@@ -807,14 +689,6 @@ gcal_manager_class_init (GcalManagerClass *klass)
                                             G_TYPE_NONE,
                                             1,
                                             GCAL_TYPE_CALENDAR);
-
-  signals[QUERY_COMPLETED] = g_signal_new ("query-completed",
-                                           GCAL_TYPE_MANAGER,
-                                           G_SIGNAL_RUN_LAST,
-                                           0,
-                                           NULL, NULL, NULL,
-                                           G_TYPE_NONE,
-                                           0);
 }
 
 static void
@@ -867,15 +741,6 @@ gcal_manager_get_calendars (GcalManager *self)
   return g_hash_table_get_values (self->clients);
 }
 
-GcalCalendar*
-gcal_manager_get_calendar_from_source (GcalManager *self,
-                                       ESource     *source)
-{
-  g_return_val_if_fail (GCAL_IS_MANAGER (self), NULL);
-
-  return g_hash_table_lookup (self->clients, source);
-}
-
 /**
  * gcal_manager_get_default_calendar:
  * @self: a #GcalManager
@@ -914,194 +779,19 @@ gcal_manager_set_default_calendar (GcalManager  *self,
 }
 
 /**
- * gcal_manager_setup_shell_search:
- * @self: a #GcalManager
- * @subscriber: an #ECalDataModelSubscriber
- *
- * Sets up the GNOME Shell search subscriber.
- */
-void
-gcal_manager_setup_shell_search (GcalManager             *self,
-                                 ECalDataModelSubscriber *subscriber)
-{
-  GTimeZone *zone;
-
-  g_return_if_fail (GCAL_IS_MANAGER (self));
-
-  if (self->shell_search_data_model)
-    return;
-
-  self->shell_search_data_model = e_cal_data_model_new (gcal_thread_submit_job);
-  g_signal_connect_object (self->shell_search_data_model,
-                           "view-state-changed",
-                           G_CALLBACK (model_state_changed),
-                           self,
-                           G_CONNECT_SWAPPED);
-
-  e_cal_data_model_set_expand_recurrences (self->shell_search_data_model, TRUE);
-
-  zone = gcal_context_get_timezone (self->context);
-  e_cal_data_model_set_timezone (self->shell_search_data_model, gcal_timezone_to_icaltimezone (zone));
-
-  self->search_view_data = g_new0 (ViewStateData, 1);
-  self->search_view_data->subscriber = subscriber;
-}
-
-/**
- * gcal_manager_set_shell_search_query:
- * @self: A #GcalManager instance
- * @query: query string (an s-exp)
- *
- * Set the query terms of the #ECalDataModel used in the shell search
- */
-void
-gcal_manager_set_shell_search_query (GcalManager *self,
-                                     const gchar *query)
-{
-  GCAL_ENTRY;
-
-  g_return_if_fail (GCAL_IS_MANAGER (self));
-
-  self->search_view_data->passed_start = FALSE;
-  self->search_view_data->search_done = FALSE;
-  self->search_view_data->sources_left = g_hash_table_size (self->clients);
-
-  if (self->search_view_data->query != NULL)
-    g_free (self->search_view_data->query);
-  self->search_view_data->query = g_strdup (query);
-
-  e_cal_data_model_set_filter (self->shell_search_data_model, query);
-}
-
-/**
- * gcal_manager_set_shell_search_subscriber:
- * @self: a #GcalManager
- * @subscriber: the #ECalDataModelSubscriber to subscribe
- * @range_start: the start of the range
- * @range_end: the end of the range
- *
- * Subscribe @subscriber to the shell data modal at the given range.
- */
-void
-gcal_manager_set_shell_search_subscriber (GcalManager             *self,
-                                          ECalDataModelSubscriber *subscriber,
-                                          time_t                   range_start,
-                                          time_t                   range_end)
-{
-  GCAL_ENTRY;
-
-  e_cal_data_model_subscribe (self->shell_search_data_model, subscriber, range_start, range_end);
-
-  GCAL_EXIT;
-}
-
-/**
- * gcal_manager_shell_search_done:
+ * gcal_manager_get_timeline:
  * @self: a #GcalManager
  *
- * Retrieves whether the search at @self is done or not.
+ * Retrieves the default timeline of GNOME Calendar.
  *
- * Returns: %TRUE if the search is finished.
+ * Returns: (transfer none): a #GcalTimeline
  */
-gboolean
-gcal_manager_shell_search_done (GcalManager *self)
+GcalTimeline*
+gcal_manager_get_timeline (GcalManager *self)
 {
-  g_return_val_if_fail (GCAL_IS_MANAGER (self), FALSE);
-
-  return self->search_view_data->search_done;
-}
-
-/**
- * gcal_manager_get_shell_search_events:
- * @self: a #GcalManager
- *
- * Retrieves all the events available for GNOME Shell search.
- *
- * Returns: (nullable)(transfer full)(content-type GcalEvent): a #GList
- */
-GList*
-gcal_manager_get_shell_search_events (GcalManager *self)
-{
-  time_t range_start, range_end;
-  GatherEventData data = {
-    .manager = self,
-    .events = NULL,
-  };
-
-  GCAL_ENTRY;
-
-  e_cal_data_model_get_subscriber_range (self->shell_search_data_model,
-                                         self->search_view_data->subscriber,
-                                         &range_start,
-                                         &range_end);
-
-  e_cal_data_model_foreach_component (self->shell_search_data_model,
-                                      range_start,
-                                      range_end,
-                                      gather_events,
-                                      &data);
-
-  GCAL_RETURN (data.events);
-}
-
-/**
- * gcal_manager_set_subscriber:
- * @self: a #GcalManager
- * @subscriber: a #ECalDataModelSubscriber
- * @range_start: the start of the range
- * @range_end: the end of the range
- *
- * Sets the @subscriber to show events between @range_start
- * and @range_end.
- */
-void
-gcal_manager_set_subscriber (GcalManager             *self,
-                             ECalDataModelSubscriber *subscriber,
-                             time_t                   range_start,
-                             time_t                   range_end)
-{
-  GCAL_ENTRY;
-
-  g_return_if_fail (GCAL_IS_MANAGER (self));
-
-  e_cal_data_model_subscribe (self->e_data_model,
-                              subscriber,
-                              range_start,
-                              range_end);
-
-  GCAL_EXIT;
-}
-
-/**
- * gcal_manager_query_client_data:
- *
- * Queries for a specific data field of the #ECalClient
- *
- * Returns: (nullable)(transfer none): a string representing
- * the retrieved value.
- */
-gchar*
-gcal_manager_query_client_data (GcalManager *self,
-                                ESource     *source,
-                                const gchar *field)
-{
-  GcalCalendar *calendar;
-  ECalClient *client;
-  gchar *out;
-
-  GCAL_ENTRY;
-
   g_return_val_if_fail (GCAL_IS_MANAGER (self), NULL);
 
-  calendar = g_hash_table_lookup (self->clients, source);
-
-  if (!calendar)
-    GCAL_RETURN (NULL);
-
-  client = gcal_calendar_get_client (calendar);
-  g_object_get (client, field, &out, NULL);
-
-  GCAL_RETURN (out);
+  return self->timeline;
 }
 
 /**
@@ -1466,92 +1156,20 @@ gcal_manager_move_event_to_source (GcalManager *self,
  *
  * Returns: (nullable)(transfer full)(content-type GcalEvent):a #GList
  */
-GList*
+GPtrArray*
 gcal_manager_get_events (GcalManager *self,
-                         ICalTime    *start_date,
-                         ICalTime    *end_date)
+                         GDateTime   *start_date,
+                         GDateTime   *end_date)
 {
-  time_t range_start, range_end;
-  GTimeZone *zone;
-  ICalTimezone *tz;
-  GatherEventData data = {
-    .manager = self,
-    .events = NULL,
-  };
+  g_autoptr (GPtrArray) events_at_range = NULL;
 
   GCAL_ENTRY;
 
   g_return_val_if_fail (GCAL_IS_MANAGER (self), NULL);
 
-  zone = gcal_context_get_timezone (self->context);
-  tz = gcal_timezone_to_icaltimezone (zone);
-  range_start = i_cal_time_as_timet_with_zone (start_date, tz);
-  range_end = i_cal_time_as_timet_with_zone (end_date, tz);
+  events_at_range = gcal_timeline_get_events_at_range (self->timeline, start_date, end_date);
 
-  e_cal_data_model_foreach_component (self->e_data_model,
-                                      range_start,
-                                      range_end,
-                                      gather_events,
-                                      &data);
-
-  GCAL_RETURN (data.events);
-}
-
-/**
- * gcal_manager_get_event_from_shell_search:
- * @self: a #GcalManager
- * @uuid: the unique identier of the event
- *
- * Retrieves the #GcalEvent with @uuid.
- *
- * Returns: (nullable)(transfer full): a #GcalEvent
- */
-GcalEvent*
-gcal_manager_get_event_from_shell_search (GcalManager *self,
-                                          const gchar *uuid)
-{
-  GcalEvent *new_event;
-  GList *l;
-  time_t range_start, range_end;
-  GatherEventData data = {
-    .manager = self,
-    .events = NULL,
-  };
-
-
-  GCAL_ENTRY;
-
-  g_return_val_if_fail (GCAL_IS_MANAGER (self), NULL);
-
-  new_event = NULL;
-
-  e_cal_data_model_get_subscriber_range (self->shell_search_data_model,
-                                         self->search_view_data->subscriber,
-                                         &range_start,
-                                         &range_end);
-
-  e_cal_data_model_foreach_component (self->shell_search_data_model,
-                                      range_start,
-                                      range_end,
-                                      gather_events,
-                                      &data);
-
-  for (l = data.events; l != NULL; l = g_list_next (l))
-    {
-      GcalEvent *event;
-
-      event = l->data;
-
-      /* Found the event */
-      if (g_strcmp0 (gcal_event_get_uid (event), uuid) == 0)
-        new_event = event;
-      else
-        g_object_unref (event);
-    }
-
-  g_list_free (data.events);
-
-  GCAL_RETURN (new_event);
+  GCAL_RETURN (g_steal_pointer (&events_at_range));
 }
 
 /**
@@ -1576,10 +1194,10 @@ gcal_manager_startup (GcalManager *self)
   GList *sources, *l;
   GError *error = NULL;
   ESourceCredentialsProvider *credentials_provider;
-  GTimeZone *zone;
 
   GCAL_ENTRY;
 
+  self->timeline = gcal_timeline_new (self->context);
   self->clients = g_hash_table_new_full ((GHashFunc) e_source_hash,
                                          (GEqualFunc) e_source_equal,
                                          g_object_unref,
@@ -1673,14 +1291,6 @@ gcal_manager_startup (GcalManager *self)
   g_signal_connect_object (self->source_registry, "source-removed", G_CALLBACK (remove_source), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (self->source_registry, "source-changed", G_CALLBACK (source_changed), self, G_CONNECT_SWAPPED);
 
-  /* create data model */
-  self->e_data_model = e_cal_data_model_new (gcal_thread_submit_job);
-
-  e_cal_data_model_set_expand_recurrences (self->e_data_model, TRUE);
-
-  zone = gcal_context_get_timezone (self->context);
-  e_cal_data_model_set_timezone (self->e_data_model, gcal_timezone_to_icaltimezone (zone));
-
   sources = e_source_registry_list_enabled (self->source_registry, E_SOURCE_EXTENSION_CALENDAR);
 
   for (l = sources; l != NULL; l = l->next)
