@@ -1,7 +1,7 @@
 /* gcal-month-view.c
  *
  * Copyright © 2015 Erick Pérez Castellanos
- *             2017 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
+ *             2017-2020 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #define G_LOG_DOMAIN "GcalMonthView"
 
 #include "config.h"
-#include "e-cal-data-model-subscriber.h"
 #include "gcal-application.h"
 #include "gcal-clock.h"
 #include "gcal-context.h"
@@ -28,6 +27,7 @@
 #include "gcal-month-cell.h"
 #include "gcal-month-popover.h"
 #include "gcal-month-view.h"
+#include "gcal-timeline-subscriber.h"
 #include "gcal-utils.h"
 #include "gcal-view.h"
 
@@ -130,8 +130,6 @@ static void          gcal_view_interface_init                    (GcalViewInterf
 
 static void          gtk_buildable_interface_init                (GtkBuildableIface  *iface);
 
-static void          e_data_model_subscriber_interface_init      (ECalDataModelSubscriberInterface *iface);
-
 static void          on_event_activated_cb                       (GcalEventWidget    *widget,
                                                                   GcalMonthView      *self);
 
@@ -143,13 +141,15 @@ static void          on_month_cell_show_overflow_popover_cb      (GcalMonthCell
                                                                   GtkWidget          *button,
                                                                   GcalMonthView      *self);
 
+static void          gcal_timeline_subscriber_interface_init     (GcalTimelineSubscriberInterface *iface);
+
 
 G_DEFINE_TYPE_WITH_CODE (GcalMonthView, gcal_month_view, GTK_TYPE_CONTAINER,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init)
-                         G_IMPLEMENT_INTERFACE (E_TYPE_CAL_DATA_MODEL_SUBSCRIBER,
-                                                e_data_model_subscriber_interface_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
-                                                gtk_buildable_interface_init));
+                                                gtk_buildable_interface_init)
+                         G_IMPLEMENT_INTERFACE (GCAL_TYPE_TIMELINE_SUBSCRIBER,
+                                                gcal_timeline_subscriber_interface_init));
 
 enum
 {
@@ -573,6 +573,16 @@ get_real_event_widget_height (GtkWidget *widget)
   return min_height;
 }
 
+static inline gboolean
+month_view_contains_event (GcalRange *month_range,
+                           GcalEvent *event)
+{
+  GcalRangeOverlap overlap;
+
+  overlap = gcal_range_calculate_overlap (month_range, gcal_event_get_range (event), NULL);
+  return overlap != GCAL_RANGE_NO_OVERLAP;
+}
+
 static void
 allocate_multiday_events (GcalMonthView *self,
                           gdouble       *vertical_cell_space,
@@ -580,12 +590,14 @@ allocate_multiday_events (GcalMonthView *self,
                           gint          *events_at_day,
                           gint          *allocated_events_at_day)
 {
+  g_autoptr (GcalRange) month_range = NULL;
   GtkAllocation event_allocation;
   GtkBorder margin;
   GList *l;
   gboolean is_rtl;
 
   is_rtl = gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL;
+  month_range = gcal_timeline_subscriber_get_range (GCAL_TIMELINE_SUBSCRIBER (self));
 
   for (l = self->multi_cell_children; l; l = g_list_next (l))
     {
@@ -603,6 +615,9 @@ allocate_multiday_events (GcalMonthView *self,
       event = gcal_event_widget_get_event (l->data);
       child_context = gtk_widget_get_style_context (l->data);
 
+      if (!month_view_contains_event (month_range, event))
+        continue;
+
       /*
        * Multiday events can "break" following these rules:
        *
@@ -744,12 +759,15 @@ allocate_single_day_events (GcalMonthView *self,
                             gint          *events_at_day,
                             gint          *allocated_events_at_day)
 {
+  g_autoptr (GcalRange) month_range = NULL;
   GHashTableIter iter;
   GtkAllocation event_allocation;
   GtkAllocation cell_allocation;
   GtkBorder margin;
   gpointer key, value;
 
+  month_range = gcal_timeline_subscriber_get_range (GCAL_TIMELINE_SUBSCRIBER (self));
+
   g_hash_table_iter_init (&iter, self->single_cell_children);
 
   while (g_hash_table_iter_next (&iter, &key, &value))
@@ -770,6 +788,7 @@ allocate_single_day_events (GcalMonthView *self,
       l = (GList*) value;
       for (aux = l; aux; aux = g_list_next (aux))
         {
+          GcalEvent *event;
           GtkStyleContext *child_context;
           GtkWidget *child_widget;
           gdouble real_height;
@@ -780,10 +799,14 @@ allocate_single_day_events (GcalMonthView *self,
           gint minimum_height;
 
           child_widget = aux->data;
+          event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (child_widget));
 
           if (!gtk_widget_get_visible (child_widget))
             continue;
 
+          if (!month_view_contains_event (month_range, event))
+            continue;
+
           child_context = gtk_widget_get_style_context (child_widget);
 
           gtk_style_context_get_margin (child_context, gtk_style_context_get_state (child_context), &margin);
@@ -1000,15 +1023,15 @@ update_month_cells (GcalMonthView *self)
                 selection_end = selection_start;
 
               /* Swap dates if end is before start */
-              if (gcal_date_time_compare_date (selection_end, selection_start) < 0)
+              if (gcal_date_time_compare_date (selection_start, selection_end) < 0)
                 {
                   GDateTime *aux = selection_end;
                   selection_end = selection_start;
                   selection_start = aux;
                 }
 
-              selected = gcal_date_time_compare_date (gcal_month_cell_get_date (cell), selection_start) >= 0 &&
-                         gcal_date_time_compare_date (gcal_month_cell_get_date (cell), selection_end) <= 0;
+              selected = gcal_date_time_compare_date (selection_start, gcal_month_cell_get_date (cell)) >= 0 &&
+                         gcal_date_time_compare_date (selection_end, gcal_month_cell_get_date (cell)) <= 0;
             }
 
           gcal_month_cell_set_selected (cell, selected);
@@ -1166,6 +1189,8 @@ gcal_month_view_set_date (GcalView  *view,
   update_header_labels (self);
   update_month_cells (self);
 
+  gcal_timeline_subscriber_range_changed (GCAL_TIMELINE_SUBSCRIBER (view));
+
   GCAL_EXIT;
 }
 
@@ -1199,32 +1224,6 @@ gcal_month_view_get_children_by_uuid (GcalView              *view,
   return filter_event_list_by_uid_and_modtype (children, mod, uuid);
 }
 
-static void
-gcal_month_view_update_subscription (GcalView *view)
-{
-  g_autoptr (GDateTime) date_start = NULL;
-  g_autoptr (GDateTime) date_end = NULL;
-  GcalMonthView *self;
-  time_t range_start;
-  time_t range_end;
-
-  self = GCAL_MONTH_VIEW (view);
-
-  g_assert (self->date != NULL);
-  date_start = g_date_time_new_local (g_date_time_get_year (self->date),
-                                      g_date_time_get_month (self->date),
-                                      1, 0, 0, 0);
-  range_start = g_date_time_to_unix (date_start);
-
-  date_end = g_date_time_add_months (date_start, 1);
-  range_end = g_date_time_to_unix (date_end);
-
-  gcal_manager_set_subscriber (gcal_context_get_manager (self->context),
-                               E_CAL_DATA_MODEL_SUBSCRIBER (self),
-                               range_start,
-                               range_end);
-}
-
 static GDateTime*
 gcal_month_view_get_next_date (GcalView *view)
 {
@@ -1251,7 +1250,6 @@ gcal_view_interface_init (GcalViewInterface *iface)
   iface->set_date = gcal_month_view_set_date;
   iface->clear_marks = gcal_month_view_clear_marks;
   iface->get_children_by_uuid = gcal_month_view_get_children_by_uuid;
-  iface->update_subscription = gcal_month_view_update_subscription;
   iface->get_next_date = gcal_month_view_get_next_date;
   iface->get_previous_date = gcal_month_view_get_previous_date;
 }
@@ -1285,118 +1283,93 @@ gtk_buildable_interface_init (GtkBuildableIface *iface)
 
 
 /*
- * ECalDataModelSubscriber interface
+ * GcalTimelineSubscriber iface
  */
 
+static GcalRange*
+gcal_month_view_get_range (GcalTimelineSubscriber *subscriber)
+{
+  g_autoptr (GDateTime) month_start = NULL;
+  g_autoptr (GDateTime) month_end = NULL;
+  GcalMonthView *self;
+
+  self = GCAL_MONTH_VIEW (subscriber);
+  month_start = g_date_time_new_local (g_date_time_get_year (self->date),
+                                       g_date_time_get_month (self->date),
+                                       1, 0, 0, 0);
+  month_end = g_date_time_add_months (month_start, 1);
+
+  return gcal_range_new (month_start, month_end, GCAL_RANGE_DEFAULT);
+}
+
 static void
-gcal_month_view_component_added (ECalDataModelSubscriber *subscriber,
-                                 ECalClient              *client,
-                                 ECalComponent           *comp)
+gcal_month_view_add_event (GcalTimelineSubscriber *subscriber,
+                           GcalEvent              *event)
 {
-  g_autoptr (GcalEvent) event = NULL;
   GcalMonthView *self;
   GcalCalendar *calendar;
   GtkWidget *event_widget;
-  GError *error;
 
-  GCAL_ENTRY;
-
-  error = NULL;
   self = GCAL_MONTH_VIEW (subscriber);
-  calendar = gcal_manager_get_calendar_from_source (gcal_context_get_manager (self->context),
-                                                    e_client_get_source (E_CLIENT (client)));
-  event = gcal_event_new (calendar, comp, &error);
-
-  if (error)
-    {
-      g_warning ("Error creating event: %s", error->message);
-      g_clear_error (&error);
-      GCAL_RETURN ();
-    }
+  calendar = gcal_event_get_calendar (event);
 
   event_widget = gcal_event_widget_new (self->context, event);
-  gcal_event_widget_set_read_only (GCAL_EVENT_WIDGET (event_widget), e_client_is_readonly (E_CLIENT (client)));
+  gcal_event_widget_set_read_only (GCAL_EVENT_WIDGET (event_widget), gcal_calendar_is_read_only (calendar));
 
   gtk_widget_show (event_widget);
   gtk_container_add (GTK_CONTAINER (subscriber), event_widget);
 
   self->pending_event_allocation = TRUE;
-
-  GCAL_EXIT;
 }
 
 static void
-gcal_month_view_component_modified (ECalDataModelSubscriber *subscriber,
-                                    ECalClient              *client,
-                                    ECalComponent           *comp)
+gcal_month_view_update_event (GcalTimelineSubscriber *subscriber,
+                              GcalEvent              *event)
 {
-  g_autoptr (GcalEvent) event = NULL;
   GcalMonthView *self;
-  GcalCalendar *calendar;
   GtkWidget *new_widget;
-  GError *error;
   GList *l;
 
   GCAL_ENTRY;
 
-  error = NULL;
   self = GCAL_MONTH_VIEW (subscriber);
-  calendar = gcal_manager_get_calendar_from_source (gcal_context_get_manager (self->context),
-                                                    e_client_get_source (E_CLIENT (client)));
-  event = gcal_event_new (calendar, comp, &error);
-
-  if (error)
-    {
-      g_warning ("Error creating event: %s", error->message);
-      g_clear_error (&error);
-      GCAL_RETURN ();
-    }
-
-  new_widget = gcal_event_widget_new (self->context, event);
 
   l = g_hash_table_lookup (self->children, gcal_event_get_uid (event));
 
-  if (l)
-    {
-      gtk_widget_destroy (l->data);
-
-      gtk_widget_show (new_widget);
-      gtk_container_add (GTK_CONTAINER (subscriber), new_widget);
-    }
-  else
+  if (!l)
     {
       g_warning ("%s: Widget with uuid: %s not found in view: %s",
-                 G_STRFUNC, gcal_event_get_uid (event),
+                 G_STRFUNC,
+                 gcal_event_get_uid (event),
                  gtk_widget_get_name (GTK_WIDGET (subscriber)));
-      gtk_widget_destroy (new_widget);
+      return;
     }
 
+  /* Destroy the old event widget (split event widgets will be destroyed too) */
+  gtk_widget_destroy (l->data);
+
+  /* Create and add the new event widget */
+  new_widget = gcal_event_widget_new (self->context, event);
+  gtk_widget_show (new_widget);
+  gtk_container_add (GTK_CONTAINER (subscriber), new_widget);
+
   self->pending_event_allocation = TRUE;
 
   GCAL_EXIT;
 }
 
 static void
-gcal_month_view_component_removed (ECalDataModelSubscriber *subscriber,
-                                   ECalClient              *client,
-                                   const gchar             *uid,
-                                   const gchar             *rid)
+gcal_month_view_remove_event (GcalTimelineSubscriber *subscriber,
+                              GcalEvent              *event)
 {
   GcalMonthView *self;
-  g_autofree gchar *uuid = NULL;
-  const gchar *sid;
+  const gchar *uuid;
   GList *l;
 
   GCAL_ENTRY;
 
   self = GCAL_MONTH_VIEW (subscriber);
-  sid = e_source_get_uid (e_client_get_source (E_CLIENT (client)));
-
-  if (rid != NULL)
-    uuid = g_strdup_printf ("%s:%s:%s", sid, uid, rid);
-  else
-    uuid = g_strdup_printf ("%s:%s", sid, uid);
-
+  uuid = gcal_event_get_uid (event);
   l = g_hash_table_lookup (self->children, uuid);
 
   if (!l)
@@ -1416,24 +1389,12 @@ gcal_month_view_component_removed (ECalDataModelSubscriber *subscriber,
 }
 
 static void
-gcal_month_view_freeze (ECalDataModelSubscriber *subscriber)
+gcal_timeline_subscriber_interface_init (GcalTimelineSubscriberInterface *iface)
 {
-}
-
-static void
-gcal_month_view_thaw (ECalDataModelSubscriber *subscriber)
-{
-}
-
-
-static void
-e_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
-{
-  iface->component_added = gcal_month_view_component_added;
-  iface->component_modified = gcal_month_view_component_modified;
-  iface->component_removed = gcal_month_view_component_removed;
-  iface->freeze = gcal_month_view_freeze;
-  iface->thaw = gcal_month_view_thaw;
+  iface->get_range = gcal_month_view_get_range;
+  iface->add_event = gcal_month_view_add_event;
+  iface->update_event = gcal_month_view_update_event;
+  iface->remove_event = gcal_month_view_remove_event;
 }
 
 
@@ -2200,6 +2161,7 @@ gcal_month_view_set_property (GObject       *object,
                                self,
                                0);
       update_weather (self, TRUE);
+      g_object_notify (object, "context");
       break;
 
     default:
