@@ -1,6 +1,6 @@
 /* gcal-week-grid.c
  *
- * Copyright (C) 2016 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
+ * Copyright (C) 2016-2020 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
  *                    Vamsi Krishna Gollapudi <pandu.sonu@yahoo.com>
  *
  * This program is free software: you can redistribute it and/or modify
@@ -43,8 +43,7 @@ static const double dashed [] =
 typedef struct
 {
   GtkWidget          *widget;
-  guint16             start;
-  guint16             end;
+  GcalEvent          *event;
 } ChildData;
 
 struct _GcalWeekGrid
@@ -83,15 +82,13 @@ static guint signals[LAST_SIGNAL] = { 0, };
 /* ChildData methods */
 static ChildData*
 child_data_new (GtkWidget *widget,
-                guint16    start,
-                guint16    end)
+                GcalEvent *event)
 {
   ChildData *data;
 
   data = g_new (ChildData, 1);
   data->widget = widget;
-  data->start = start;
-  data->end = end;
+  data->event = g_object_ref (event);
 
   return data;
 }
@@ -121,65 +118,6 @@ destroy_event_widget (GcalWeekGrid *self,
 }
 
 /* Auxiliary methods */
-static void
-get_event_range (GcalWeekGrid *self,
-                 GcalEvent    *event,
-                 guint16      *start,
-                 guint16      *end)
-{
-  GDateTime *week_start;
-  GTimeSpan diff;
-  gboolean week_start_dst;
-
-  if (!self->active_date)
-    return;
-
-  week_start = gcal_date_time_get_start_of_week (self->active_date);
-  week_start_dst = g_date_time_is_daylight_savings (week_start);
-
-  if (start)
-    {
-      GDateTime *event_start;
-      gboolean event_start_dst;
-
-      event_start = g_date_time_to_local (gcal_event_get_date_start (event));
-      event_start_dst = g_date_time_is_daylight_savings (event_start);
-
-      diff = g_date_time_difference (event_start, week_start);
-
-      *start = CLAMP (diff / G_TIME_SPAN_MINUTE, 0, MAX_MINUTES);
-      *start += 60 * (event_start_dst - week_start_dst);
-
-      g_clear_pointer (&event_start, g_date_time_unref);
-    }
-
-  if (end)
-    {
-      g_autoptr (GDateTime) inclusive_end_date = NULL;
-      g_autoptr (GDateTime) event_end = NULL;
-      gboolean event_end_dst;
-
-      inclusive_end_date = g_date_time_add_seconds (gcal_event_get_date_end (event), -1);
-      event_end = g_date_time_to_local (inclusive_end_date);
-      event_end_dst = g_date_time_is_daylight_savings (event_end);
-
-      diff = g_date_time_difference (event_end, week_start);
-
-      *end = CLAMP (diff / G_TIME_SPAN_MINUTE, 0, MAX_MINUTES);
-      *end += 60 * (event_end_dst - week_start_dst);
-
-      /*
-       * XXX: it may happen that the event has the same start and end
-       * dates. For this case, just enforce that the event is at least
-       * 1 minute long.
-       */
-      if (start && *start == *end)
-        *end = *end + 1;
-    }
-
-  g_clear_pointer (&week_start, g_date_time_unref);
-}
-
 static inline gint
 uint16_compare (gconstpointer a,
                 gconstpointer b)
@@ -189,14 +127,13 @@ uint16_compare (gconstpointer a,
 
 static inline guint
 get_event_index (GcalRangeTree *tree,
-                 guint16        start,
-                 guint16        end)
+                 GcalRange     *range)
 {
   g_autoptr (GPtrArray) array;
   gint idx, i;
 
   i = idx = 0;
-  array = gcal_range_tree_get_data_at_range (tree, start, end);
+  array = gcal_range_tree_get_data_at_range (tree, range);
 
   if (!array)
     return 0;
@@ -216,47 +153,30 @@ get_event_index (GcalRangeTree *tree,
 
 static guint
 count_overlaps_at_range (GcalRangeTree *self,
-                         guint16        start,
-                         guint16        end)
+                         GcalRange     *range)
 {
-  guint64 i, counter;
+  g_autoptr (GDateTime) start = NULL;
+  g_autoptr (GDateTime) end = NULL;
+  guint64 counter;
+  gint minutes;
+  gint i;
 
   g_return_val_if_fail (self, 0);
-  g_return_val_if_fail (end >= start, 0);
 
   counter = 0;
+  start = gcal_range_get_start (range);
+  end = gcal_range_get_end (range);
+  minutes = g_date_time_difference (end, start) / G_TIME_SPAN_MINUTE;
 
-  for (i = start; i < end; i++)
+  for (i = 0; i < minutes; i++)
     {
+      g_autoptr (GcalRange) minute_range = NULL;
       guint n_events;
 
-      n_events = gcal_range_tree_count_entries_at_range (self, i, i + 1);
-
-      if (n_events == 0)
-        break;
-
-      counter = MAX (counter, n_events);
-    }
-
-  return counter;
-}
-
-static guint
-count_overlaps_of_event (GcalRangeTree *self,
-                         guint16        day_start,
-                         guint16        day_end,
-                         guint16        event_start,
-                         guint16        event_end)
-{
-  guint64 i, counter;
-
-  counter = count_overlaps_at_range (self, event_start, day_end);
-
-  for (i = event_start; i > day_start; i--)
-    {
-      guint n_events;
-
-      n_events = gcal_range_tree_count_entries_at_range (self, i - 1, i);
+      minute_range = gcal_range_new_take (g_date_time_add_minutes (start, i),
+                                          g_date_time_add_minutes (start, i + 1),
+                                          GCAL_RANGE_DEFAULT);
+      n_events = gcal_range_tree_count_entries_at_range (self, minute_range);
 
       if (n_events == 0)
         break;
@@ -303,14 +223,13 @@ gcal_week_grid_forall (GtkContainer *container,
   guint i;
 
   self = GCAL_WEEK_GRID (container);
-  widgets_data = gcal_range_tree_get_data_at_range (self->events, 0, MAX_MINUTES);
+  widgets_data = gcal_range_tree_get_all_data (self->events);
 
-  for (i = 0; widgets_data && i < widgets_data->len; i++)
+  for (i = 0; i < widgets_data->len; i++)
     {
       ChildData *data;
 
       data = g_ptr_array_index (widgets_data, i);
-
       callback (data->widget, callback_data);
     }
 
@@ -607,6 +526,7 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
                               GtkAllocation *allocation)
 {
   GcalWeekGrid *self = GCAL_WEEK_GRID (widget);
+  g_autoptr (GDateTime) week_start = NULL;
   GcalRangeTree *overlaps;
   gboolean ltr;
   gdouble minutes_height;
@@ -636,29 +556,35 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
   /* Temporary range tree to hold positioned events' indexes */
   overlaps = gcal_range_tree_new ();
 
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
+
   /*
    * Iterate through weekdays; we don't have to worry about events that
    * jump between days because they're already handled by GcalWeekHeader.
    */
   for (i = 0; i < 7; i++)
     {
+      g_autoptr (GcalRange) day_range = NULL;
       GPtrArray *widgets_data;
-      guint16 day_start;
-      guint16 day_end;
       guint j;
 
-      day_start = i * MINUTES_PER_DAY;
-      day_end = day_start + MINUTES_PER_DAY;
-      widgets_data = gcal_range_tree_get_data_at_range (self->events, day_start, day_end);
+      day_range = gcal_range_new_take (g_date_time_add_days (week_start, i),
+                                       g_date_time_add_days (week_start, i + 1),
+                                       GCAL_RANGE_DEFAULT);
+      widgets_data = gcal_range_tree_get_data_at_range (self->events, day_range);
 
       for (j = 0; widgets_data && j < widgets_data->len; j++)
         {
           GtkStyleContext *context;
           GtkAllocation child_allocation;
           GtkWidget *event_widget;
+          GcalRange *event_range;
+          GDateTime *event_start;
+          GDateTime *event_end;
           ChildData *data;
           GtkBorder margin;
           guint64 events_at_range;
+          gint event_minutes;
           gint natural_height;
           gint widget_index;
           gint offset;
@@ -667,13 +593,16 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
 
           data =  g_ptr_array_index (widgets_data, j);
           event_widget = data->widget;
+          event_range = gcal_event_get_range (data->event);
+          event_start = gcal_event_get_date_start (data->event);
+          event_end = gcal_event_get_date_end (data->event);
           context = gtk_widget_get_style_context (event_widget);
 
           /* The total number of events available in this range */
-          events_at_range = count_overlaps_of_event (self->events, day_start, day_end, data->start, data->end);
+          events_at_range = count_overlaps_at_range (self->events, event_range);
 
           /* The real horizontal position of this event */
-          widget_index = get_event_index (overlaps, data->start, data->end);
+          widget_index = get_event_index (overlaps, event_range);
 
           /* Gtk complains about that */
           gtk_widget_get_preferred_height (event_widget, NULL, &natural_height);
@@ -683,10 +612,11 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
                                         gtk_style_context_get_state (context),
                                         &margin);
 
+          event_minutes = g_date_time_difference (event_end, event_start) / G_TIME_SPAN_MINUTE;
           width = column_width / events_at_range - margin.left - margin.right;
-          height = (data->end - data->start) * minutes_height - margin.top - margin.bottom;
+          height = event_minutes * minutes_height - margin.top - margin.bottom;
           offset = (width + margin.left + margin.right) * widget_index;
-          y = (data->start % MINUTES_PER_DAY) * minutes_height + margin.top;
+          y = (g_date_time_get_hour (event_start) * 60 + g_date_time_get_minute (event_start)) * minutes_height + margin.top;
 
           if (ltr)
             x = column_width * i + offset + allocation->x + margin.left + 1;
@@ -710,10 +640,7 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
            * Add the current event to the temporary overlaps tree so we have a way to
            * know how many events are already positioned in the current column.
            */
-          gcal_range_tree_add_range (overlaps,
-                                     data->start,
-                                     data->end,
-                                     GUINT_TO_POINTER (widget_index));
+          gcal_range_tree_add_range (overlaps, event_range, GUINT_TO_POINTER (widget_index));
         }
 
       g_clear_pointer (&widgets_data, g_ptr_array_unref);
@@ -1149,13 +1076,9 @@ gcal_week_grid_add_event (GcalWeekGrid *self,
                           GcalEvent    *event)
 {
   GtkWidget *widget;
-  guint16 start, end;
 
   g_return_if_fail (GCAL_IS_WEEK_GRID (self));
 
-  end = 0;
-  start = 0;
-
   g_object_ref (event);
 
   widget = g_object_new (GCAL_TYPE_EVENT_WIDGET,
@@ -1164,12 +1087,9 @@ gcal_week_grid_add_event (GcalWeekGrid *self,
                          "orientation", GTK_ORIENTATION_VERTICAL,
                          NULL);
 
-  get_event_range (self, event, &start, &end);
-
   gcal_range_tree_add_range (self->events,
-                             start,
-                             end,
-                             child_data_new (widget, start, end));
+                             gcal_event_get_range (event),
+                             child_data_new (widget, event));
 
   setup_event_widget (self, widget);
   gtk_widget_show (widget);
@@ -1186,14 +1106,12 @@ gcal_week_grid_remove_event (GcalWeekGrid *self,
 
   g_return_if_fail (GCAL_IS_WEEK_GRID (self));
 
-  widgets = gcal_range_tree_get_data_at_range (self->events, 0, MAX_MINUTES);
+  widgets = gcal_range_tree_get_all_data (self->events);
 
   for (i = 0; widgets && i < widgets->len; i++)
     {
       ChildData *data;
       GcalEvent *event;
-      guint16 event_start;
-      guint16 event_end;
 
       data = g_ptr_array_index (widgets, i);
       event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (data->widget));
@@ -1201,9 +1119,7 @@ gcal_week_grid_remove_event (GcalWeekGrid *self,
       if (g_strcmp0 (gcal_event_get_uid (event), uid) != 0)
         continue;
 
-      get_event_range (self, event, &event_start, &event_end);
-
-      gcal_range_tree_remove_range (self->events, data->start, data->end, data);
+      gcal_range_tree_remove_range (self->events, gcal_event_get_range (event), data);
       destroy_event_widget (self, data->widget);
       gtk_widget_queue_allocate (GTK_WIDGET (self));
       g_object_unref (event);
@@ -1227,7 +1143,7 @@ gcal_week_grid_get_children_by_uuid (GcalWeekGrid          *self,
 
   events = NULL;
   result = NULL;
-  widgets = gcal_range_tree_get_data_at_range (self->events, 0, MAX_MINUTES);
+  widgets = gcal_range_tree_get_all_data (self->events);
 
   for (i = 0; widgets && i < widgets->len; i++)
     {
