@@ -22,6 +22,7 @@
 #include "gcal-clock.h"
 #include "gcal-context.h"
 #include "gcal-debug.h"
+#include "gcal-timeline-subscriber.h"
 #include "gcal-year-view.h"
 #include "gcal-view.h"
 #include "gcal-utils.h"
@@ -126,14 +127,16 @@ enum {
 };
 
 static void          gcal_view_interface_init                    (GcalViewInterface  *iface);
-static void          gcal_data_model_subscriber_interface_init   (ECalDataModelSubscriberInterface *iface);
+
 static void          update_weather                              (GcalYearView       *self);
 
+static void          gcal_timeline_subscriber_interface_init     (GcalTimelineSubscriberInterface *iface);
+
 
 G_DEFINE_TYPE_WITH_CODE (GcalYearView, gcal_year_view, GTK_TYPE_BOX,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init)
-                         G_IMPLEMENT_INTERFACE (E_TYPE_CAL_DATA_MODEL_SUBSCRIBER,
-                                                gcal_data_model_subscriber_interface_init));
+                         G_IMPLEMENT_INTERFACE (GCAL_TYPE_TIMELINE_SUBSCRIBER,
+                                                gcal_timeline_subscriber_interface_init));
 
 static gint
 compare_events (GcalEventWidget *widget1,
@@ -575,32 +578,6 @@ update_sidebar_headers (GtkListBoxRow *row,
   g_clear_pointer (&row_date, g_date_time_unref);
 }
 
-static void
-update_date (GcalYearView *year_view,
-             GDateTime    *new_date)
-{
-  gboolean needs_reset = FALSE;
-
-  if (year_view->date &&
-      year_view->start_selected_date &&
-      !g_date_time_equal (year_view->date, new_date))
-    {
-      needs_reset = TRUE;
-    }
-
-  gcal_set_date_time (&year_view->date, new_date);
-
-  year_view->first_week_of_year = get_last_week_of_year_dmy (year_view->first_weekday,
-                                                             1, G_DATE_JANUARY,
-                                                             g_date_time_get_year (year_view->date));;
-  year_view->last_week_of_year = get_last_week_of_year_dmy (year_view->first_weekday,
-                                                            31, G_DATE_DECEMBER,
-                                                            g_date_time_get_year (year_view->date));
-
-  if (needs_reset)
-    reset_sidebar (year_view);
-}
-
 static void
 calculate_coord_for_date (GcalYearView *year_view,
                           gint          day,
@@ -1588,6 +1565,158 @@ navigator_drag_leave_cb (GcalYearView   *self,
   gtk_widget_queue_draw (navigator);
 }
 
+/*
+ * GcalTimelineSubscriber implementation
+ */
+
+static GcalRange*
+gcal_year_view_get_range (GcalTimelineSubscriber *subscriber)
+{
+  g_autoptr (GDateTime) start = NULL;
+  g_autoptr (GDateTime) end = NULL;
+  GcalYearView *self;
+
+  self = GCAL_YEAR_VIEW (subscriber);
+  start = g_date_time_new_local (g_date_time_get_year (self->date), 1, 1, 0, 0, 0);
+  end = g_date_time_add_years (start, 1);
+
+  return gcal_range_new (start, end, GCAL_RANGE_DEFAULT);
+}
+
+static void
+gcal_year_view_add_event (GcalTimelineSubscriber *subscriber,
+                          GcalEvent              *event)
+{
+  GcalYearView *self;
+  GDateTime *event_start;
+  GDateTime *event_end;
+  guint start_month;
+  guint end_month;
+  guint i;
+
+  GCAL_ENTRY;
+
+  self = GCAL_YEAR_VIEW (subscriber);
+
+  g_debug ("Caching event '%s' in Year view", gcal_event_get_uid (event));
+
+  event_start = gcal_event_get_date_start (event);
+  event_end = gcal_event_get_date_end (event);
+
+  /* Calculate the start & end months */
+  start_month = g_date_time_get_month (event_start) - 1;
+  end_month = g_date_time_get_month (event_end) - 1;
+
+  if (g_date_time_get_year (event_start) < g_date_time_get_year (self->date))
+    start_month = 0;
+
+  if (g_date_time_get_year (event_end) > g_date_time_get_year (self->date))
+    end_month = 11;
+
+  /* Add the event to the cache */
+  for (i = start_month; i <= end_month; i++)
+    g_ptr_array_add (self->events[i], g_object_ref (event));
+
+  update_sidebar (self);
+
+  gtk_widget_queue_draw (GTK_WIDGET (self->navigator));
+
+  GCAL_EXIT;
+}
+
+static void
+gcal_year_view_remove_event (GcalTimelineSubscriber *subscriber,
+                             GcalEvent              *event)
+{
+  g_autoptr (GList) children = NULL;
+  GcalYearView *self;
+  GList *l;
+  const gchar *event_id;
+  guint n_children;
+  guint i;
+
+  GCAL_ENTRY;
+
+  self = GCAL_YEAR_VIEW (subscriber);
+  event_id = gcal_event_get_uid (event);
+
+  n_children = 0;
+  children = gtk_container_get_children (GTK_CONTAINER (self->events_sidebar));
+
+  for (l = children; l != NULL; l = g_list_next (l))
+    {
+      GcalEventWidget *child_widget;
+      GcalEvent *list_event;
+
+      child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
+      list_event = gcal_event_widget_get_event (child_widget);
+
+      if (g_strcmp0 (event_id, gcal_event_get_uid (list_event)) == 0)
+        gtk_widget_destroy (GTK_WIDGET (l->data));
+      else
+        n_children++;
+    }
+
+  /*
+   * No children left visible, all the events were removed and now we have to show the
+   * 'No Events' placeholder.
+   */
+  if (n_children == 0)
+    {
+      update_no_events_page (self);
+      gtk_stack_set_visible_child_name (GTK_STACK (self->navigator_stack), "no-events");
+    }
+
+  /* Also remove from the cached list of events */
+  for (i = 0; i < 12; i++)
+    {
+      GPtrArray *events;
+      guint j;
+
+      events = self->events[i];
+
+      for (j = 0; j < events->len; j++)
+        {
+          GcalEvent *cached_event;
+
+          cached_event = g_ptr_array_index (events, j);
+
+          if (!g_str_equal (gcal_event_get_uid (cached_event), event_id))
+            continue;
+
+          g_debug ("Removing event '%s' from Year view's cache", event_id);
+          g_ptr_array_remove (events, event);
+        }
+    }
+
+  gtk_widget_queue_draw (GTK_WIDGET (self->navigator));
+
+  GCAL_EXIT;
+}
+
+static void
+gcal_year_view_update_event (GcalTimelineSubscriber *subscriber,
+                             GcalEvent              *event)
+{
+  GCAL_ENTRY;
+
+  gcal_year_view_remove_event (subscriber, event);
+  gcal_year_view_add_event (subscriber, event);
+
+
+  GCAL_EXIT;
+}
+
+static void
+gcal_timeline_subscriber_interface_init (GcalTimelineSubscriberInterface *iface)
+{
+  iface->get_range = gcal_year_view_get_range;
+  iface->add_event = gcal_year_view_add_event;
+  iface->remove_event = gcal_year_view_remove_event;
+  iface->update_event = gcal_year_view_update_event;
+}
+
+
 /* GcalView implementation */
 static GDateTime*
 gcal_year_view_get_date (GcalView *view)
@@ -1601,9 +1730,36 @@ static void
 gcal_year_view_set_date (GcalView  *view,
                          GDateTime *date)
 {
+  GcalYearView *self;
+  gboolean year_changed;
+  gboolean needs_reset;
+
   GCAL_ENTRY;
 
-  update_date (GCAL_YEAR_VIEW (view), date);
+  self = GCAL_YEAR_VIEW (view);
+
+  needs_reset = self->date &&
+                self->start_selected_date &&
+                !g_date_time_equal (self->date, date);
+
+  year_changed = !self->date || g_date_time_get_year (self->date) != g_date_time_get_year (date);
+
+  gcal_set_date_time (&self->date, date);
+
+  self->first_week_of_year = get_last_week_of_year_dmy (self->first_weekday,
+                                                        1,
+                                                        G_DATE_JANUARY,
+                                                        g_date_time_get_year (self->date));
+  self->last_week_of_year = get_last_week_of_year_dmy (self->first_weekday,
+                                                       31,
+                                                       G_DATE_DECEMBER,
+                                                       g_date_time_get_year (self->date));
+
+  if (needs_reset)
+    reset_sidebar (self);
+
+  if (year_changed)
+    gcal_timeline_subscriber_range_changed (GCAL_TIMELINE_SUBSCRIBER (view));
 
   GCAL_EXIT;
 }
@@ -1626,28 +1782,6 @@ gcal_year_view_get_children_by_uuid (GcalView              *view,
   return result;
 }
 
-static void
-gcal_year_view_update_subscription (GcalView *view)
-{
-  g_autoptr (GDateTime) date_start = NULL;
-  g_autoptr (GDateTime) date_end = NULL;
-  GcalYearView *self;
-  time_t range_start;
-  time_t range_end;
-
-  self = GCAL_YEAR_VIEW (view);
-  date_start = g_date_time_new_local (g_date_time_get_year (self->date), 1, 1, 0, 0, 0);
-  range_start = g_date_time_to_unix (date_start);
-
-  date_end = g_date_time_add_years (date_start, 1);
-  range_end = g_date_time_to_unix (date_end);
-
-  gcal_manager_set_subscriber (gcal_context_get_manager (self->context),
-                               E_CAL_DATA_MODEL_SUBSCRIBER (self),
-                               range_start,
-                               range_end);
-}
-
 static GDateTime*
 gcal_year_view_get_next_date (GcalView *view)
 {
@@ -1670,11 +1804,9 @@ gcal_year_view_get_previous_date (GcalView *view)
 static void
 gcal_view_interface_init (GcalViewInterface *iface)
 {
-  /* FIXME: implement what's needed */
   iface->get_date = gcal_year_view_get_date;
   iface->set_date = gcal_year_view_set_date;
   iface->get_children_by_uuid = gcal_year_view_get_children_by_uuid;
-  iface->update_subscription = gcal_year_view_update_subscription;
   iface->get_next_date = gcal_year_view_get_next_date;
   iface->get_previous_date = gcal_year_view_get_previous_date;
 }
@@ -1688,10 +1820,9 @@ gcal_year_view_finalize (GObject *object)
   g_free (year_view->navigator_grid);
   g_free (year_view->selected_data);
 
-  g_free (year_view->start_selected_date);
-  g_free (year_view->end_selected_date);
-
-  g_clear_pointer (&year_view->date, g_free);
+  g_clear_pointer (&year_view->start_selected_date, g_date_time_unref);
+  g_clear_pointer (&year_view->end_selected_date, g_date_time_unref);
+  g_clear_pointer (&year_view->date, g_date_time_unref);
 
   g_clear_object (&year_view->calendar_settings);
 
@@ -1876,167 +2007,6 @@ gcal_year_view_direction_changed (GtkWidget        *widget,
     year_view->k = 1;
 }
 
-static void
-gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
-                                ECalClient              *client,
-                                ECalComponent           *comp)
-{
-  g_autoptr (GcalEvent) event = NULL;
-  GcalCalendar *calendar;
-  GcalYearView *self;
-  GDateTime *event_start, *event_end;
-  GError *error;
-  guint i, start_month, end_month;
-
-  GCAL_ENTRY;
-
-  error = NULL;
-  self = GCAL_YEAR_VIEW (subscriber);
-  calendar = gcal_manager_get_calendar_from_source (gcal_context_get_manager (self->context),
-                                                    e_client_get_source (E_CLIENT (client)));
-  event = gcal_event_new (calendar, comp, &error);
-
-  if (error)
-    {
-      g_warning ("Error creating event: %s", error->message);
-      g_clear_error (&error);
-      GCAL_RETURN ();
-    }
-
-  g_debug ("Caching event '%s' in Year view", gcal_event_get_uid (event));
-
-  event_start = gcal_event_get_date_start (event);
-  event_end = gcal_event_get_date_end (event);
-
-  /* Calculate the start & end months */
-  start_month = g_date_time_get_month (event_start) - 1;
-  end_month = g_date_time_get_month (event_end) - 1;
-
-  if (g_date_time_get_year (event_start) < g_date_time_get_year (self->date))
-    start_month = 0;
-
-  if (g_date_time_get_year (event_end) > g_date_time_get_year (self->date))
-    end_month = 11;
-
-  /* Add the event to the cache */
-  for (i = start_month; i <= end_month; i++)
-    g_ptr_array_add (self->events[i], g_object_ref (event));
-
-  update_sidebar (self);
-
-  gtk_widget_queue_draw (GTK_WIDGET (self->navigator));
-
-  GCAL_EXIT;
-}
-
-static void
-gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
-                                  ECalClient              *client,
-                                  const gchar             *uid,
-                                  const gchar             *rid)
-{
-  GcalYearView *year_view;
-  GList *children, *l;
-  ESource *source;
-  g_autofree gchar *uuid = NULL;
-  guint i;
-  gint number_of_children;
-
-  GCAL_ENTRY;
-
-  year_view = GCAL_YEAR_VIEW (subscriber);
-  source = e_client_get_source (E_CLIENT (client));
-  if (rid != NULL)
-    uuid = g_strdup_printf ("%s:%s:%s", e_source_get_uid (source), uid, rid);
-  else
-    uuid = g_strdup_printf ("%s:%s", e_source_get_uid (source), uid);
-
-  children = gtk_container_get_children (GTK_CONTAINER (year_view->events_sidebar));
-  number_of_children = g_list_length (children);
-
-  for (l = children; l != NULL; l = g_list_next (l))
-    {
-      GcalEventWidget *child_widget;
-      GcalEvent *event;
-
-      child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
-      event = gcal_event_widget_get_event (child_widget);
-
-      if (g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
-        {
-          gtk_widget_destroy (GTK_WIDGET (l->data));
-          number_of_children--;
-        }
-    }
-
-  /*
-   * No children left visible, all the events were removed and now we have to show the
-   * 'No Events' placeholder.
-   */
-  if (number_of_children == 0)
-    {
-      update_no_events_page (year_view);
-      gtk_stack_set_visible_child_name (GTK_STACK (year_view->navigator_stack), "no-events");
-    }
-
-  /* Also remove from the cached list of events */
-  for (i = 0; i < 12; i++)
-    {
-      GPtrArray *events;
-      guint j;
-
-      events = year_view->events[i];
-
-      for (j = 0; j < events->len; j++)
-        {
-          GcalEvent *event;
-
-          event = g_ptr_array_index (events, j);
-
-          if (!g_str_equal (gcal_event_get_uid (event), uuid))
-            continue;
-
-          g_debug ("Removing event '%s' from Year view's cache", uuid);
-          g_ptr_array_remove (events, event);
-        }
-    }
-
-  gtk_widget_queue_draw (GTK_WIDGET (year_view->navigator));
-
-  g_list_free (children);
-
-  GCAL_EXIT;
-}
-
-static void
-gcal_year_view_component_changed (ECalDataModelSubscriber *subscriber,
-                                  ECalClient              *client,
-                                  ECalComponent           *comp)
-{
-  ECalComponentId *id;
-
-  GCAL_ENTRY;
-
-  id = e_cal_component_get_id (comp);
-
-  gcal_year_view_component_removed (subscriber, client, e_cal_component_id_get_uid (id), e_cal_component_id_get_rid (id));
-  gcal_year_view_component_added (subscriber, client, comp);
-
-  g_clear_pointer (&id, e_cal_component_id_free);
-
-  GCAL_EXIT;
-}
-
-static void
-gcal_year_view_freeze (ECalDataModelSubscriber *subscriber)
-{
-  ;
-}
-
-static void
-gcal_year_view_thaw (ECalDataModelSubscriber *subscriber)
-{
-}
 
 static void
 gcal_year_view_class_init (GcalYearViewClass *klass)
@@ -2131,15 +2101,6 @@ gcal_year_view_init (GcalYearView *self)
                      GDK_ACTION_MOVE);
 }
 
-static void
-gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
-{
-  iface->component_added = gcal_year_view_component_added;
-  iface->component_modified = gcal_year_view_component_changed;
-  iface->component_removed = gcal_year_view_component_removed;
-  iface->freeze = gcal_year_view_freeze;
-  iface->thaw = gcal_year_view_thaw;
-}
 
 static void
 update_weather (GcalYearView *self)
